package parse_du2_speedup_logs;

import java.io.*;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Du2Parser {
	
	
	static final String versions[] = { "serial", "parallel_c", "parallel_p" /*, "optimized"*/ };
	
	
	static void ReadFile(String fn, String par_version, SortedMap<TaskIdentifier, TaskResults> data) 
			throws NumberFormatException, IOException {
		
		
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(fn)));
		
		String line;
		
		TaskIdentifier ti = new TaskIdentifier(0,0,0);
		String current_version = "serial";
		
		while((line = br.readLine()) != null) {
			
			/* New task */
			if(line.startsWith("Trying")) {
				
				/* example: Trying input_size 1024, cluster_count 2, iteration_count 64. */
				Pattern p = Pattern.compile("\\d+");
				Matcher m = p.matcher(line);
				int[] tiData = new int[3];
				int pos = 0;
				while (m.find() && pos < 3) {
			        tiData[pos] = Integer.parseInt(m.group());
			        pos++;
			    }
				
				ti = new TaskIdentifier(tiData[0], tiData[1], tiData[2]);
				
				if(!data.containsKey(ti)) {
					data.put(ti, new TaskResults(ti, versions));
				}
			}
			
			if(line.startsWith("Serial")) {
				current_version = "serial";
			}
			
			if(line.startsWith("Parallel")) {
				current_version = par_version;
			}
			
			if(line.startsWith("real")) {
				Pattern p = Pattern.compile("(\\d+)m(\\d+\\.\\d+)s");
				Matcher m = p.matcher(line);
				/*while(m.find()) {
					System.out.println(m.group(1));
					System.out.println(m.group(2));
				}*/
				m.find();
				int minutes = Integer.parseInt(m.group(1));
				double seconds = Double.parseDouble(m.group(2));
				data.get(ti).AddVersionResult(current_version, minutes*60 + seconds);
			}
		}
		
		br.close();
		
	}
	
	
	public static void main(String[] args) throws IOException {
		
		SortedMap<TaskIdentifier, TaskResults> data = new TreeMap<TaskIdentifier, TaskResults>();
		
		ReadFile("speedup_c.log", "parallel_c", data);
		ReadFile("speedup_p.log", "parallel_p", data);
		
		writeCsv(data);
		
	}
	
	
	static void writeCsv(SortedMap<TaskIdentifier, TaskResults> data) throws IOException {
		BufferedWriter wr = new BufferedWriter(new FileWriter("speedup.csv"));
		
		for(TaskResults taskResults : data.values()) {
			wr.write(taskResults.toString());
			wr.write("\n");
		}
		
		wr.close();
	}

}
